﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Windows.Forms.VisualStyles;

namespace VidyaVitamin.Plugin.Thumbnail
{
	public partial class SliderBox : UserControl
	{
		private Rectangle track_rc = new Rectangle();
		private Rectangle thumb_rc = new Rectangle();

		private bool thumbclicked = false;
		private TrackBarThumbState thumbstate = TrackBarThumbState.Normal;

		public SliderBox()
		{
			spos = 128;
			this.BackColor = Color.Transparent;
			this.DoubleBuffered = true;
		}

		private const int twidth = 14;
		private int spos, min = 48, max = 256;
		public int Value
		{
			get { return spos; }
			set
			{
				spos = value;
				if (min > spos) spos = min;
				else if (max < spos) spos = max;

				UpdateThumbRect();
				Invalidate();
			}
		}

		private int step = 10;
		public int SliderStep
		{
			get { return step; }
			set { step = value; }
		}

		public event EventHandler ValueChanged;

		protected override void OnClientSizeChanged(EventArgs e)
		{
			base.OnClientSizeChanged(e);
			UpdateThumbRect();
		}

		private void UpdateThumbRect()
		{
			Rectangle rc = ClientRectangle;
			rc.X = twidth;
			rc.Width -= twidth * 2;

			int tw = 0;
			double a = (double)(tw - (rc.Right - tw)) / (double)(min - max);
			thumb_rc.X = (int)(a * (double)(spos - min)) + tw;
			thumb_rc.Y = 2;
			thumb_rc.Size = new Size(twidth, rc.Height - 4);

			if (ValueChanged != null) ValueChanged(this, new EventArgs());
		}

		protected override void OnEnter(EventArgs e)
		{
			thumbstate = TrackBarThumbState.Hot;
			Invalidate();
		}

		protected override void OnLeave(EventArgs e)
		{
			thumbstate = TrackBarThumbState.Normal;
			Invalidate();
		}

		protected override void OnMouseDown(MouseEventArgs e)
		{
			if (thumb_rc.Contains(e.Location)) {
				thumbclicked = true;
				thumbstate = TrackBarThumbState.Pressed;
			} else {
				thumbstate = TrackBarThumbState.Hot;

				int tx = thumb_rc.X + thumb_rc.Width / 2;
				if (tx < e.X) {
					spos += step;
				} else if (tx > e.X) {
					spos -= step;
				} else {
					thumbstate = TrackBarThumbState.Pressed;
					thumbclicked = true;
				}

				UpdateThumbRect();
			}

			Invalidate();
		}

		protected override void OnMouseMove(MouseEventArgs e)
		{
			if (thumbclicked == true) {
				Rectangle rc = ClientRectangle;
				int xpos = e.X;
				if (xpos < rc.Left) spos = min;
				else if (xpos > rc.Right) spos = max;
				else {
					double a = (double)(min - max) / (double)(rc.Left - rc.Right);
					spos = (int)(a * (double)(xpos - rc.Left)) + min;
					Parent.Text = spos.ToString();
				}

				UpdateThumbRect();
			}

			Invalidate();
		}

		protected override void OnMouseUp(MouseEventArgs e)
		{
			if (thumbclicked == true) {
				if (e.Location.X > track_rc.X &&
					e.Location.X < (track_rc.Right - thumb_rc.Width)) {
					thumbclicked = false;
					thumbstate = TrackBarThumbState.Hot;
					Invalidate();
				}

				thumbclicked = false;
			}
		}

		protected override void OnPaint(PaintEventArgs e)
		{
			base.OnPaint(e);

			Graphics g = e.Graphics;

			// 背景の台形を描画
			Rectangle rc = ClientRectangle;
			Point[] pts = new Point[3];
			pts[0] = new Point(twidth / 2, rc.Height / 2 + 2);
			pts[1] = new Point(twidth / 2, rc.Height / 2);
			pts[2] = new Point(rc.Right - twidth / 2, rc.Height / 2 - 4);
			g.DrawLines(SystemPens.ControlDark, pts);

			Point[] pts2 = new Point[3];
			pts2[0] = pts[2];
			pts2[1] = new Point(rc.Right - twidth / 2, rc.Height / 2 + 2);
			pts2[2] = pts[0];
			g.DrawLines(SystemPens.ControlLightLight, pts2);

			if (TrackBarRenderer.IsSupported) {
				// 水平バーのトラックを描画
				TrackBarRenderer.DrawHorizontalTrack(g, track_rc);
				// つまみを描画
				TrackBarRenderer.DrawBottomPointingThumb(g, thumb_rc, thumbstate);
			} else {
				ControlPaint.DrawBorder(g, track_rc,
					SystemColors.Control, ButtonBorderStyle.Inset);
				ControlPaint.DrawButton(g, thumb_rc, ButtonState.Normal);
			}
		}
	}

	[System.Windows.Forms.Design.ToolStripItemDesignerAvailability(
	   System.Windows.Forms.Design.ToolStripItemDesignerAvailability.All)]
	public class ToolStripTrackBar : ToolStripControlHost
	{
		public ToolStripTrackBar() : base(new SliderBox())
		{
			this.AutoSize = false;
			this.Width = 80;
		}

		public SliderBox TrackBar
		{
			get { return Control as SliderBox; }
		}

		protected override void OnOwnerChanged(EventArgs e)
		{
			base.OnOwnerChanged(e);
			this.Height = this.Owner.Height;
		}

		protected override void OnSubscribeControlEvents(Control control)
		{
			base.OnSubscribeControlEvents(control);

			SliderBox sb = (SliderBox)control;
			sb.ValueChanged += new EventHandler(OnSBValueChanged);
		}

		protected override void  OnUnsubscribeControlEvents(Control control)
		{
			base.OnUnsubscribeControlEvents(control);

			SliderBox sb = (SliderBox)control;
			sb.ValueChanged -= new EventHandler(OnSBValueChanged);
		}

		public event EventHandler ValueChanged;

		private void OnSBValueChanged(object sender, EventArgs e)
		{
			if (ValueChanged != null) ValueChanged(this, e);
		}

		public int Value
		{
			get { return ((SliderBox)Control).Value; }
			set { ((SliderBox)Control).Value = value; }
		}
	}
}
